using System.Collections.Generic;

using AntMe.English;


//Insert a slashdot behind AntMe.Player followed by your name (please don't use blanks).
//It could look like this: "AntMe.Player.NickBoesl".
namespace AntMe.Player.gogokefakefa
{
    //Using the 'player'-attribute you can specify the name of your ant colony and the 
    //first and last name of the player. In order for the game to find your ant colony you
    //have to name your colony; inserting your name 
    [Player(
        ColonyName = "gogokefakefa",
        FirstName = "Georgi",
        LastName = "Georgiev"
        )]

    //The 'caste'-attribute makes it possible to change the properties of your ants. You can make 
    //them faster or give them more energy etc. But be careful: Only '-1', '0', '1' and '2' can be
    //be used and the sum of all properties of one caste has to be '0'!
    //To use this attribute you have to name the caste fist. After that you can return the name of that
    //type by using 'ChooseType'.
    //For further information how to use the ant-types see chapter 6 of the tutorial. 
    [Caste(
        Name = "Carrier",
        SpeedModificator = 2,
        LoadModificator = 2,
        AttackModificator = -1,
        EnergyModificator = -1,
        RangeModificator = 0,
        ViewRangeModificator = -1,
        RotationSpeedModificator = -1
        )]
    [Caste(
    Name = "Scout",
    SpeedModificator = 2,
    LoadModificator = -1,
    AttackModificator = -1,
    EnergyModificator = -1,
    RangeModificator = 0,
    ViewRangeModificator = 2,
    RotationSpeedModificator = -1
         )]

    public class kefakefaAnts : BaseAnt
    {
        #region Castes

        /// <summary>
        /// Chooses a specific caste.
        /// </summary>
        /// <param name="typeCount">List of available ants from caste</param>
        /// <returns>Name of chosen caste</returns>
        public override string ChooseType(Dictionary<string, int> typeCount)
        {
            if (typeCount["Scout"] * 1024 > typeCount["Carrier"])
                return "Carrier";
            return "Scout";
        }

        #endregion

        #region Movement


        /// <summary>
        /// Is called periodically everytime the ant does not know where to go.
        /// </summary>
        public override void Waits()
        {
            GoAhead();
        }

        /// <summary>
        /// Is called once when the ant has exceeded a third of it's maximum range.
        /// </summary>
        public override void BecomesTired()
        {
        }

        #endregion

        #region Food

        /// <summary>
        /// Is called periodically everytime the ant spots at least one heap of sugar.
        /// </summary>
        /// <param name="sugar">The closest heap of sugar.</param>
        public override void Spots(Sugar sugar)
        {
            if (CurrentLoad == 0 && !(Target is Bug))
            {
                GoToTarget(sugar);
            }
        }

        /// <summary>
        /// Is called periodically everytime the ant spots at least one piece of fruit.
        /// </summary>
        /// <param name="fruit">The closest piece of fruit.</param>
        public override void Spots(Fruit fruit)
        {
            if (CurrentLoad == 0 && NeedsCarrier(fruit))
            {
                GoToTarget(fruit);
            }
        }

        /// <summary>
        /// Is called once when a sugar heap is the ant's target and it reaches the sugar.
        /// </summary>
        /// <param name="sugar">The heap of sugar.</param>
        public override void TargetReached(Sugar sugar)
        {
            int distance, direction;
            distance = Coordinate.GetDistanceBetween(this, sugar);
            direction = Coordinate.GetDegreesBetween(this, sugar);
            MakeMark(direction, distance);
            Take(sugar);
            GoBackToAnthill();
        }

        /// <summary>
        /// Is called once when a piece of fruit is the ant's target and it reaches the fruit.
        /// </summary>
        /// <param name="fruit">The piece of fruit.</param>
        public override void TargetReached(Fruit fruit)
        {
            if (NeedsCarrier(fruit))
            {
                int distance, direction;
                distance = Coordinate.GetDistanceBetween(this, fruit);
                direction = Coordinate.GetDegreesBetween(this, fruit);
                MakeMark(direction, distance);
                Take(fruit);
                GoBackToAnthill();
            }
        }

        #endregion

        #region Communication

        /// <summary>
        /// Is called once when the ant 'smells' a marker from its own ant colleagues.
        /// A marker can only be 'smelled' once and will never be smelled again.
        /// </summary>
        /// <param name="marker">Closest new marker.</param>
        public override void SmellsFriend(Marker marker)
        {
            if (Target == null)
            {
                TurnToDirection(marker.Information);
                GoAhead();
            }
        }

        /// <summary>
        /// Is called repeatedly each time at least one other ant from the same colony is spotted.
        /// </summary>
        /// <param name="ant">The closest friendly ant.</param>
        public override void SpotsFriend(Ant ant)
        {
        }

        /// <summary>
        /// Is called, when the ant spots a friendly/allied ant from another team.
        /// </summary>
        /// <param name="ant"></param>
        public override void SpotsPartner(Ant ant)
        {
        }

        #endregion

        #region Fight

        /// <summary>
        /// Is called repeatedly each time the ant spots at least one bug.
        /// </summary>
        /// <param name="bug">The closest bug.</param>
        public override void SpotsEnemy(Bug bug)
        {
        }

        /// <summary>
        /// Is called repeatedly each time the ant spots at least one ant of another colony.
        /// </summary>
        /// <param name="ant">The closest inimical ant.</param>
        public override void SpotsEnemy(Ant ant)
        {
            if (CurrentLoad > 0)
                GoBackToAnthill();
        }

        /// <summary>
        /// Is called repeatedly each time the ant is attacked by a bug.
        /// </summary>
        /// <param name="bug">Closest attacking bug.</param>
        public override void UnderAttack(Bug bug)
        {
            //  MakeMark(0, 30);
            //   Attack(bug);
            if (CurrentEnergy < MaximumEnergy / 4)
                GoBackToAnthill();
        }

        /// <summary>
        /// Is called repeatedly each time the ant is attacked by another ant.
        /// </summary>
        /// <param name="ant">The closest inimical attacking ant.</param>
        public override void UnderAttack(Ant ant)
        {
            if (CurrentEnergy < MaximumEnergy / 4)
                GoBackToAnthill();
        }

        #endregion

        #region Misc

        /// <summary>
        /// Is called when the ant has just died.
        /// </summary>
        /// <param name="kindOfDeath">The reason for the ant's death.</param>
        public override void HasDied(KindOfDeath kindOfDeath)
        {
        }

        /// <summary>
        /// Is called each simulation step and not inflicted by5 any other external influence.
        /// </summary>
        public override void Tick()
        {
            if (Target is Anthill && CurrentLoad > 0 && CarringFruit == null)
            {
                MakeMark(Direction + 180);
            }

            if (CarringFruit != null && NeedsCarrier(CarringFruit))
                MakeMark(0, 10);
        }

        #endregion
    }
}